#pragma once
/************ros 相关头文件*******/
/* #include "std_msgs/Float64MultiArray.h"
#include <nav_msgs/Path.h> //示例：rviz的path显示头文件
#include <ros/callback_queue.h> */

/************使用的库*******/
#include <sstream>
#include <iostream>
#include <chrono>
#include <vector>
// #include <Eigen/Core>
// #include <Eigen/LU>
// #include <boost/thread/thread.hpp>

/************自定义的头文件*******/
// #include <can_recorder/Can.h>         //示例：CEM_MSG_INPUT
// #include <LatCtrl32/control_Input.h>  //示例：
// #include <LatCtrl32/control_Output.h> //示例：

namespace ADC
{
namespace common
{
using namespace std;

#define TIMERSTART(tag) auto tag##_start = std::chrono::steady_clock::now(), tag##_end = tag##_start
#define TIMEREND(tag) tag##_end = std::chrono::steady_clock::now()

#define DURATION_s(tag) std::chrono::duration_cast<std::chrono::seconds>(tag##_end - tag##_start).count()
#define DURATION_ms(tag) std::chrono::duration_cast<std::chrono::milliseconds>(tag##_end - tag##_start).count()
#define DURATION_us(tag) std::chrono::duration_cast<std::chrono::microseconds>(tag##_end - tag##_start).count()
#define DURATION_ns(tag) std::chrono::duration_cast<std::chrono::nanoseconds>(tag##_end - tag##_start).count()

#define BEHAVIORANALYZERCYCLE 50 // ms
#define SMINFOSUBCYCLE 100 //Hz
#define RMINFOSUBCYCLE 25 //Hz
#define VSEINFOSUBCYCLE 100 //Hz
#define EHRINFOSUBCYCLE 20 //Hz
#define INTERNALVARCYCLE 10 //Hz

// calibration variables applied in BA are defined below
const float k_BA_TotalDurationTimeThreshold = 2.0; // sec
const int k_BA_SamplePointNumber = 20;
const float c_BA_ms2kmh = 3.6;
const float k_BA_CrrdrLenUpThr = 80.0;
const float k_BA_CrrdrLenLowThr = 40.0;
const float k_BA_TmGapLvl0 = 1.5; // 0613
const float k_BA_TmGapLvl1 = 1.8; // 0613
const float k_BA_TmGapLvl2 = 2.1; // 0613
const float k_BA_TmGapLvlx = 2.1;

// BNA Calibration
const vector<float> k_BA_BNACostDecayGainTb = {1, 0.5, 0.25, 0.15, 0.05};
const vector<float> k_BA_BNACostDecayTmTb = {0.0, 5.0, 10.0, 15.0, 20.0};
// BNA BA Cost Ref
const float k_BA_BNAChgCostCommon = 0.3;
const float k_BA_BNAChgCost_S2 = 0.7;     // need demarchate with PD
const float k_BA_BNAChgCost_S3_encrgA = 0.2;
const float k_BA_BNAChgCost_S3_encrgB = 0.1; // ####0.1
const float k_BA_BNAChgCost_S3_pnshA = 0.5;
const float k_BA_BNAChgCost_S4_encrgA = 0.2;
const float k_BA_BNAChgCost_S4_pnshA = 0.5;
const float k_BA_BNAChgCost_S4_pnshB = 0.6;

const float k_BA_BNACommonChgCost_encrgA = 0.2;
const float k_BA_BNACommonChgCost_pnshA = 0.5;
const float k_BA_BNACommonChgCost_pnshB = 0.6;
const float k_BA_BNACommonChgCost_pnshC = 0.7;
const float k_BA_BNACommonChgCost_nutrl = 0.3;
const float k_BA_BNAALCCourage_A = 0.25;
const float k_BA_BNAChgCostSteady_EncrgA = -0.2;
const float k_BA_BNAChgCostSteady_EncrgB = -0.3;
const float k_BA_BNAChgCostSteady_EncrgC = -0.4;

const float k_BA_BNAEachLyrChgCostWeight = 0.7;
const vector<float> k_BA_BNAQuantizationCommon = {0,1};
const vector<float> k_BA_BNAQuantizationCommon_re = {1,0};
const bool k_BA_BNALCPFrtPnsh = 0; //For LCP FrtSpace Punish ref to Bck Space
const float k_BA_BNALCPFrtPnshGain = 0.05;
const bool  k_BA_BNABAChgCostFrtLCPSpdLmOn = 1;
const float k_BA_BNABAChgCostFrtLCPSpdLmDistThr = 200;
const float k_BA_BNABAChgCostFrtLCPSpdLmSpdErrThr = 3.33;  //12kph

//RMUpate 0831
extern bool k_RMU_LCPUseILCCorridor;

// LCP Iner 0512
const bool k_BA_BNABAChgCostLCPInerOn = 1;

// ILC Ref 0512
const bool k_BA_BNABAChgCostILCOn = 1;
const float k_BA_BNABAChgCostILCEncrg = 0.15; // chg 0 - > 0.15 0815 make sure cost lower than lcp 
const float k_BA_BNABAChgCostILCPnsh = 0.2;

// NA PD Cost Ref
// Soft Button
const bool k_BA_BNAPDClsnIN = 1; //20220408
const bool k_BA_BNAPDCostIN = 1; //20220323   turn on 0712
const bool k_BA_BNAPDCostDelimitOff = 1;  //ADD 0722  Turn off delimit 
const bool k_BA_BNAPDCostNorRskOff = 0; //0712
const bool k_BA_BNAPDCostUnexpectRskOn_EgoBck = 1; //0714
extern bool k_BA_BNAPDCostUnexpectRskOn_ALL;

// Delimit
const float k_BA_BNADelimitLgtAccelISO = 5.0;
const float k_BA_BNADelimitLgtAccelISO_re = -4.0;
const float k_BA_BNADelimitLatAccelThr = 3.0;
const float k_BA_BNADelimitLatAccelThr_re = -3.0;
const float k_BA_BNAStnObjLatDistMin = 0.5;
const float k_BA_BNADelimitValue = 1.0;
const float k_BA_BNADelimitClsnGain = 10.0; // Clsn Pnsh Gain

// Unexpect
extern float k_BA_BNAUnexpectObjPnshCost; // 0706 For uninteresting obj which be estimated in danger condition, ADD Big Punishment cost | 0714 value 2->4 | 0811 4-> 8
const bool k_BA_BNAUnexpectObjPnshOn = 1;    // trigger & para chg by zhiyang 0712
const float k_BA_BNAUnexpectObjTTCThr = 2.5; // TTC TD D para need be demarcated 0706
const float k_BA_BNAUnexpectObjTDThr = 0.5;   // 0.6 ->0.5 byPD 0815
const float k_BA_BNAUnexpectObjDisThr = 6;

// Normal Comfrt
const float k_BA_BNANormalSpdEffAvgWgt = 0.5; // Adjust Normal EffPart AvgSpd WEIGHT

const float k_BA_BNANormalCfrtLgtAccelWgt = 0.4;
const float k_BA_BNANormalCfrtLgtJrkWgt = 0.3;
const float k_BA_BNANormalCfrtLatAccelWgt = 0.2;
const float k_BA_BNANormalCfrtLatJrkWgt = 0.1; // 4 part Wgtadd equat 1;
const vector<float> k_BA_BNALatAccelSection = {0,3};
const vector<float> k_BA_BNALatJerkSection = {0,2};
const vector<float> k_BA_BNALgtAccelSection = {0,5};
const vector<float> k_BA_BNALgtJerkSection = {0,5};

// Normal Risk
const float k_BA_BNANormalRskTTCDgrThr = 2.5;
const float k_BA_BNANormalRskTTCLFDgrThr = 2.0;
const float k_BA_BNANormalRskTTCUnDgrCost = 0.0;
extern float k_BA_BNANormalRskTTCDgrCost; // 0804: 0.5-->0.1 0811: set to 8.0
const float k_BA_BNANormalRskTTCLFUnDgrCost  = 0.0f; // 0503: 0.1-->0
const float k_BA_BNANormalRskLFTmDistThr = 0.45;
const float k_BA_BNANormalRskLCTmDistThr = 0.5; // 0.6 -> 0.5 byPD 0815
const float k_BA_BNANormalRskLCPTmDistThr = 0.5; // 0.6 -> 0.5 byPD 0815
const float k_BA_BNANormalRskLFDistThr = 6.0;
const float k_BA_BNANormalRskLCDistThr = 6.0;
const float k_BA_BNANormalRskLCPDistThr = 6.0;  // Normal para chg base on zhiyang0801   LC&LCP Dist DO NOT UPDATE || LC&LCP Dist chg 4->6 By PD 0803

// Unexpect & Normal Risk NOR_Part
const float k_BA_BNANormalRskTTCDgrThr_NOR = 2.5; //0803 TTC TD D para use for TTC == 999 scnario in LC ,provided by PD 
const float k_BA_BNANormalRskLCTmDistThr_NOR = 0.3;
const float k_BA_BNANormalRskLCDistThr_NOR = 5.2; // 0817: 5.5 -> 5.2 from pd

const float k_BA_BNAUnexpectObjDistThr_NOR = 5.2; // 0817: 5.5 -> 5.2 from pd
const float k_BA_BNAUnexpectObjTDThr_NOR = 0.3;
const float k_BA_BNANormalRskTTCNORThr = 5; // 6 -> 5 by pd 0816

const float k_BA_BNAUnexpectObjTTCThr_NOR_EgoBck = 9;  //0810 Using special para for unexpect egobckobj by PD  
const float k_BA_BNAUnexpectObjDistThr_NOR_EgoBck = 5.5;
const float k_BA_BNAUnexpectObjTDThr_NOR_EgoBck = 0.45;

// PDCost Value in BNA Calc Ref
const float k_BA_BNANormalPDNormalFctry_Eff = 0.0;
const float k_BA_BNANormalPDNormalFctry_Cmfrt = 0.0;
const float k_BA_BNANormalPDNormalFctry_Rsk = 1.0;

const float k_BA_BNARiskLgtDistGainLo = 0.5;
const float k_BA_BNARiskLgtDistGainHi = 1.2;
const float k_BA_BNARiskLgtDistGainReLo = 0.8;
const float k_BA_BNARiskLgtDistGainReHi = 1.0;

const float k_BA_BNARiskFrtObjSfDistTmGain = 0.4;
const float k_BA_BNARiskRearObjSfDistTmGain = 1.0;
const float k_BA_BNARiskSfD0 = 2.0;

// LCG Calibration
const bool k_BA_LCGDrveMdTestOn = false;
const vector<float> k_BA_LCGDsrdDistLv1 = {1.0, 1.0, 1.2, 1.25, 1.25, 1.3, 1.3, 1.3, 1.25, 1.2, 1.2, 1.2, 1.2, 1.2};
const vector<float> k_BA_LCGDsrdDistLv1Spd = {0.0, 0.001, 2.77, 5.55, 8.33, 11.11, 13.89, 16.67, 19.44, 22.22, 25.0, 27.78, 33.33, 41.67};
const bool k_BA_LCGLCPSetLmtOn = 1;
const float k_BA_LCGDeftTgtDist = 3.0; // use for protecting calc tar&ego lne dis 0624

// trp dist lvl
extern vector<float> k_BA_LCGEstDistSpd;    // m/s 0611
const vector<float> k_BA_LCGEstDistLv0 = {2.8, 9.0, 14.8, 22.0, 45.0, 51.0};
const vector<float> k_BA_LCGEstDistLv1 = {3.0, 10.0, 16.0, 30.0, 55.0, 61.0};
extern vector<float> k_BA_LCGEstDistLv2;
extern vector<float> k_BA_LCGEstDistLv3;  // chg 20m/s  lv3  23->20  0622 merge Lgt   // chg to ori follow distance  0623

const vector<float> k_BA_LCGEstDistLv0_bck = {2.8, 9.0, 16.8, 32.0, 45.0, 51.0};
const vector<float> k_BA_LCGEstDistLv1_bck = {3.0, 10.0, 18.0, 40.0, 55.0, 61.0};
extern vector<float> k_BA_LCGEstDistLv2_bck;    // lv
extern vector<float> k_BA_LCGEstDistLv3_bck;  // 0708 add bck dist table, chg in 10m/s with 2m to 20m/s with 6m

extern float k_BA_BNAEgoLneFctr; // add 0623 ref to lgt para  0.6  0.6>0.55 refer to 0704 chg log for LC egoLne 
extern float k_BA_BNATarLneFctr; // add 0623 ref to lgt para  0.5  0708chg 0.45 to 0.5 before newlogic insert
extern float k_BA_BNALcpEgoLneFctr_frt; // use for lcp egolane sfy space calc add 0623    both frt&bck use same ego timedis, bck is more sfy. || chg 0.2 -> 0.1 -> 0.13 For more oppotunity
extern float k_BA_BNALcpEgoLneFctr_bck;
//LC
const bool k_BA_LCGLinkTrpDist = true;  // transfer to TRPLgt Est Dist Mode if be true
const float k_BA_LCGAlcEgoSfyDis = 4.0;
const float k_BA_LCGAlcEgoTmDis = 0.55;
const float k_BA_LCGAlcEgoSfyDisRe = 3.0;
const float k_BA_LCGALCEgoTmDisRe = 0.35;
const float k_BA_LCGAlcTarSfyDis = 4.0;
const float k_BA_LCGAlcTarTmDis = 0.55;
const float k_BA_LCGAlcTarSfyDisRe = 3.0;
const float k_BA_LCGALCTarTmDisRe = 0.35;
const float k_BA_LCGAlcPDDisDecrease = 1.0;
extern vector<float> k_BA_LCGAlcTarBckReGainSpd; //Tb ADD 0712
extern vector<float> k_BA_LCGAlcTarBckReGainTb;  // use for enlarge mid spd debounce  merge with trplgt

extern vector<float> k_BA_LCGAlcAbltTorGainTb; //0719  Use for reducing ReAlcDistance to aviod Abolition
extern vector<float> k_BA_LCGAlcAbltTorC0;
extern float k_BA_LCGAlcAbltTorGainLo;
extern float k_BA_LCGAlcAbltTorGainHi;
extern bool k_BA_LCGAlcAbltTorOff;

extern float k_BA_LCGALCSRLv1Gain_Bsic; // 0720 ADD Drvring Style Basic Gain || 0728 1.3 ->1.2
extern float k_BA_LCGALCSRLv2Gain_Bsic; // Normal& Default   || 0805 reduce normal style from 1.0 -> 0.95
extern float k_BA_LCGALCSRLv3Gain_Bsic; // chg from 0.8 -> 0.7 -> 0.72 to enhance performance 0725
const bool k_BA_LCGALCSRIn = 1; 

const float k_BA_LCGLCFrtTTC = 6.0;
const float k_BA_LCGLCFrtReTTC = 3.5;
const float k_BA_LCGLCBckTTC = 6.2;
const float k_BA_LCGLCBckReTTC = 3.5;

const float k_BA_LCGLCPEnrgGain = 0.9;

// link with trp 0414
extern float k_BA_LCGAlcEgoReGain;
extern float k_BA_LCGLCLCPFlowGain;
//LCP
const float k_BA_LCGEgoLenth = 7.6;  // chg from 4.9 -> 7.6 to make sure frt safety
const float k_BA_LCGLCPSftyDist = 1.5;

const float k_BA_LCGLCPFrtRgFrtSfyDis = 4.0;
const float k_BA_LCGLCPFrtRgFrtTmDis = 0.55;
const float k_BA_LCGLCPFrtRgBckSfyDis = 4.0;
const float k_BA_LCGLCPFrtRgBckTmDis = 0.55;  
const float k_BA_LCGLCPBckRgFrtSfyDis = 4.0;
const float k_BA_LCGLCPBckRgFrtTmDis = 0.55;
const float k_BA_LCGLCPBckRgBckSfyDis = 4.0;
const float k_BA_LCGLCPBckRgBckTmDis = 0.55; //above 8 para same with LC

const float k_BA_LCGLCPDisDbSrtGain = 1.2;
const float k_BA_LCGLCPDisDbWrkInGain = 1.0;

const float k_BA_LCGLCPArvEstEgoAccelSpd = 3.33; //12kph    20->12kph  0525
const float k_BA_LCGLCPArvEstEgoDeAccelSpd = 4.16; //15kph
const float k_BA_LCGLCPAccelTmThr = 8.0; // 0713 5->8 for Test
const float k_BA_LCGLCPDeAccelTmThr = 5.0;
const float k_BA_LCGLCPSpdUpTest = 18.0;
const float k_BA_LCGLCPSpdDnTest = 15.0; // only for lcp test
extern float k_BA_LCGLCPBckScNrObjSpdThr; // 32kph  target spd low than this para, LCP CAN NOT GEN   0518
extern float k_BA_LCGLCPBckScNrObjSpdErrThr; //20kph    EgoSpd&NrSpd Err bigger than 20kph    need quit LCP Gen 0617
extern float k_BA_LCGLCSpdErrSftyThr_frt;    //45kph    || 45 -> 55kph  0729
extern float k_BA_LCGLCSpdErrSftyThr_bck;    //35kph    || 35 -> 45kph  0729
extern float k_BA_LCGLCSpdErrSftyThr_refrt;  //45kph    || 45 -> 60kph  0729
extern float k_BA_LCGLCSpdErrSftyThr_rebck;  //40kph    || 40 -> 50kph  0729

const float k_BA_LCGLCPFrtSpdErrCdThr = 5.0;//18kph  frt lcp  ego&nr spd err thr 0525
const float k_BA_LCGArvDistPrtctVal = 0.1; // in case that arv dist small than zero  0525

const bool k_BA_LCGLCPBckDynArvEstTmOn = 1;   // to cover high spd lcp_bck   satisfy with res4. 0611   (above 3 para)
extern vector<float> k_BA_LCGLCPBckArvEstTmThrSpdFctr;    // convert m/s    0 40 80 120 160 kph
extern vector<float> k_BA_LCGLCPBckDynArvTm; // 0617 added  | 5 6 7 8 9 -> 6 8 10 10 11 
const bool k_BA_LCGLCPDyaSfDis = 1; // if false use constant sfy dis (default 2)  for LCP Ego Lne Sfy Space Calc   0623

const float k_BA_LCGLCPFrtLCPFrtDisSpdErrThr = 4.16; //15kph when ego accel to frtlcp,Ego & NrFrt Spd Err low than this para, FrtLcpDis_TarLne_EgoFrt use small one's spd to Estimate
const bool k_BA_LCGDistCmPrsOn = true;
extern vector<float> k_BA_LCGDistCmprsSpdErrFctr;    // spderr m/s    

extern vector<float> k_BA_LCGDistCmprsGain;   // 0705 decrease compress distance || ADD conservative aggressive table for FrtDist Cmpress & Recmpress  //0720
extern vector<float> k_BA_LCGDistCmprsGain_Cnsrv;
extern vector<float> k_BA_LCGDistCmprsGain_Agrsv;
extern vector<float> k_BA_LCGDistReCmprsGain; // 0705 add recmrps for sfty 0708 enhance 3m/s err gain 1.15->1.2   // ||0729 reduce
extern vector<float> k_BA_LCGDistReCmprsGain_Cnsrv;
extern vector<float> k_BA_LCGDistReCmprsGain_Agrsv;// ||0729 reduce

const float k_BA_LCGBckObjSpdErrSftyThr = 8.0; // 0711 WORK For huge Spd Err between Ego & BckObj in LC LCP Manner; In case of danger LC and LCP_Bck scnario abolition.

const bool k_BA_LCGBckCmprs2DOn = true;
extern vector<float> k_BA_LCGBckCmprsSpdErr;
extern vector<float> k_BA_LCGBckCmprsEgoSpd;
extern vector<float> k_BA_LCGBckCmprsTb; // above add 0711  replace 1-DTable test in bckCmprs
                                                                                                                                                     // chg in 0712  reduce 40-80spd low err gain
const float k_BA_LCGLCPEgoLneSpcDbnc = 1.15;    //Add 0802 For reduce bck lcp abolition in EgoLneSpaceCondition || chg from 1.5 -> 1.15 use as deboune
const float k_BA_LCGLCPBckNrObjSpdThr = 12.5;
const float k_BA_LCGLCPBckNrObjAccThr1 = -0.9;
const float k_BA_LCGLCPBckNrObjAccThr2 = -0.7;

const float k_BA_LCGLCPActLonThr = 3.0;  // chg from 10 -> 3 just for TEST
const float k_BA_LCGLCPActTmCtHi = 600.0;
const vector<float> k_BA_LCGMrgSpdErr = {0.0, 3.0, 6.0};
const vector<float> k_BA_LCGMrgSpdErrFctr_frt = {0.8, 0.7, 0.5};
const vector<float> k_BA_LCGMrgSpdErrFctr_bck = {0.9, 0.8, 0.6};
// PassageGen
const float k_BA_LCGEgoLaneD0 = 3.0;
const float k_BA_LCGSideLaneD0 = 2.5;
const float k_BA_LCGSideLaneTau = 0.4;
const float k_BA_LCGYellowGain = 0.8;
const float k_BA_LCGRedGain = 0.4;

const float k_BA_DistBtwVehFrt2BackAxis = 3.8;
const float k_BA_LCGEgoLneEndDistThr = 150.0;

// EHR related below
const float k_BA_LastLCPntTH = 5.0f; // sec 2.0
extern float k_BA_DesLCPntTH1st; // sec 50
extern float k_BA_DesLCPntTH2nd; // sec 50
extern float k_BA_DesLCPntTH3rd; // sec 30
extern float k_BA_DesLCPntTH4th; // sec 30
extern float k_BA_DesLCPntTH1st_B; // sec gap 10
extern float k_BA_DesLCPntTH2nd_B; // sec gap 10
extern float k_BA_DesLCPntTH3rd_B; // sec gap 10
extern float k_BA_DesLCPntTH4th_B; // sec gap 5
extern float k_BA_DesLCPntLowThr1st; // m
extern float k_BA_DesLCPntLowThr2nd; // m 1300
extern float k_BA_DesLCPntLowThr3rd; // m 1600
extern float k_BA_DesLCPntLowThr4th; // m 1800

const int k_BA_HistVehInfoLstSize = 40;
const float k_BA_HistVehInfoSmplCycle = 1.0f; // sec 1.0
// const float k_BA_LgTermFastLnHistSmplPntSize = 30;
const float k_BA_VelLowerPerc = 0.30f; // 0.25
const int k_BA_LgTermFastLnHistSmplPntThrd = 30; // 20
const int k_BA_LgTermFastLnSumSmplPntNum = 30;

// const float k_BA_LgTermSlowLnHistSmplPntSize = 40;
const int k_BA_LgTermSlowLnHistSmplPntThrd = 20;
const int k_BA_LgTermSlowLnSumSmplPntNum = 40; 

const float k_BA_EHRMsgStopUpdtThr = 1.0f; // sec
const float k_BA_LaneEndCalcDistThr = 300.0f; // m
const float k_BA_LaneEndCalcTHThr = 30.0f; // sec

const float k_BA_GlobalPathFllwCostCoeff = 0.6f;
const float k_BA_LaneEndCostCoeff = 0.4f;
const float k_BA_LgtTermFastLnCostCoeff = 0.1f;
const float k_BA_LgtTermSlowLnCostCoeff = 0.2f;
const float k_BA_TrffcEffcCostCoeff = 0.3f;
const float k_BA_TrgtLnCostCoeff = 0.1f;

const float k_BA_GlobalCostCoeff = 0.7f;

extern float k_BA_SpprLCDistThrd01; // m 1200
extern float k_BA_SpprLCDistThrd02; // m 1500

const vector<float> k_BA_UnstsfyDgrPercLstActv = {0.9, 0.8, 0.7, 0.6};
const vector<float> k_BA_UnstsfyDgrPercLstInactv = {0.95, 0.85, 0.75, 0.65};
const vector<float> k_BA_UnstsfyDgrDurTmLst01 = {72.0, 36.0, 18.0, 12.0}; // {72, 36, 18, 12};
const vector<float> k_BA_UnstsfyDgrDurTmLst02 = {25.0, 12.0, 10.0, 7.0}; // {40, 15, 10, 10}
const vector<float> k_BA_UnstsfyDgrDurTmLst03 = {20.0, 11.0, 9.0, 7.0}; // {10, 8, 7, 5};
extern float k_BA_EffcTrffcCostSpdPerc;
extern float k_BA_LnEndTHThr; // 3.0f
extern float k_BA_LnEndTHThr01; // 3.0f
const int k_BA_UnstsfDgrFlgCntThrd = 500; // ms
extern float k_BA_VehFlowCalcMaxRng; // m 200 -> 250
const int k_BA_OptResDebCnt = 4;
const bool k_BA_OptResDebProcFlg = true;
extern float k_BA_DrcnIndAheadOnDistThrd; // m
extern float k_BA_LineFllwDrcnIndOnDurThr; // sec
const float k_BA_MaxDrcnIndOnDurTm = 20.0f; // sec
const float k_BA_MaxLnChgAblDurTm = 4.5f; // sec 20220527: 3.5 --> 4.5
const float k_BA_MaxDstFltrSpdLmtInfo = 800.0f; // m
const float k_BA_DrcnIndFrcdOnAheadDst = 300.0f; // m
extern float k_BA_MaxLnChgAblNotDrvCmdDur; // sec
const float k_BA_AblObjDispTH = 0.5f; // sec
extern float k_BA_LaneEndOffsetSpprThr; // m
const float k_BA_LCIntervalDstThr = 600.0f; // m
const float k_BA_DynIntervalChgInRmpMrgCARd = 30.0f; // sec
extern float k_BA_LCIntervalLvl01; // sec
const int k_BA_TORStsDebCnt = 10;
const float k_BA_MinNgbrLnStRng = 5.0f; // m 
const float k_BA_LftApprLnBndryLowThr = 1.02f; // m original 1.0
const float k_BA_LftApprLnBndryHighThr = 1.15f; // m original 1.15
const float k_BA_RghtApprLnBndryLowThr = -1.02f; // m original -1.0
const float k_BA_RghtApprLnBndryHighThr = -1.15f; // m original -1.15
const float k_BA_ILCCrrdC0DiffThr = 0.05f;
const float k_BA_MaxFuncOnDurTmThr = 20000.0f;
const float k_BA_MinWaitTmLCAbl2LF = 1.5f; // sec
const bool k_BA_InclLaneExtCrrd = true;
const float k_BA_AutoLCSpdHighThr = 45.0f; // km/h
const float k_BA_AutoLCSpdLowThr = 35.0f; // km/h 0818: 30.0 --> 35.0
const float k_BA_AntiDrctLCDist2WPThr = 800.0f; // m
extern float k_BA_AntiDrctLCDist2LastLCPntThr_Aggr; // m aggressive threshold
extern float k_BA_AntiDrctLCDist2LastLCPntThr_Cnsr; // m conservative threshold
const float k_BA_MinShineHghtlghtSpdGap = 1.0f; // m/s
const float k_BA_MinShineHghtlghtTH = 0.0f; // sec
const float k_BA_LastLCPntSetSpd01 = 90.0f; // kph
const float k_BA_MinDstToMrgRdAllwLC = 1300.0f; // m
const float k_BA_ActAdjSetSpdThr = 200.0f; // m
const float k_BA_MinVehFlowDense = 3.0f; // 2.5
const uint8_t k_BA_MinAntiDrctLCNum = 0; // set to 0 for line-follow way 
const float k_BA_LastLnEndSetSpd01 = 60.0f; // m
const float k_BA_MrgRdSpdLmt01 = 98.0f; //kph
const float k_BA_MrgRdSpdLmtDstThr01 = 200.0f; // m
const float k_BA_MrgRdSpdLmt02 = 78.0f; //kph
const float k_BA_LastLCPntSetSpd02 = 64.0f; //kph
const float k_BA_MrgRdSpdLmtDstThr02 = 100.0f; // m
const float k_BA_MinDstLnEndLCAblCncl = 15.0f; // m
const float k_BA_MinLastLCPntDst = 160.0f; // m 150
const float k_BA_MinEgoLnWidthThr = 2.8f; // m
extern float k_BA_MinCoolDur4LCRmpMrgCARd; // 0.5s
const bool k_BA_DrctIndOnInAdvFlg = true; // determine whether to light direction indicator in advance in merge case
const bool k_BA_DlyLnChgInOptionalCase = true; // determine whether to delay sending lane change ref line to TRPLat to realize light direction indicator ahead of time
const float k_BA_DrctIndOnInAdvTm = 1.2f; // sec
const float k_BA_DrctIndOnInAdvTm02 = 0.8f; // sec
const int k_BA_DrcnIndReqDebCnt = 8; // 5 
const bool k_BA_PrepareObjToSMFlg = false; // determine whether to execute six object ids calculation for SM
const bool k_BA_SpprLC2MrgRdLn = true; // determine whether to overpass slow car by the merge road lane in aggressive drive mode
const float k_BA_SpprLC2RmdWPExtLnDstThr = 1000.0f;
extern float k_BA_MinLaneEndLCPntDst; // m the mimimum dist to lane end point for lane ending takeover request, no matter what speed is
extern float k_BA_MinLaneEndLCPntDst01; // m the mimimum dist to lane end point for lane ending takeover request, no matter what speed is
const float k_BA_MinLaneEndLCPntDst02 = 120.0f; // m only dist to lane end point is less than this threshold, lane ending takeover request is permitted
const int k_BA_FlowDenseDebCnt = 60; // lane vehicle flow density calc debounce counter
const int k_BA_SpdCondDebCnt = 20; // lane vehicle flow speed condition debounce counter
const int k_BA_SpdCondDebCnt_Aggr = 5; // lane vehicle flow speed condition debounce counter for aggressive style
extern float k_BA_DrcnIndAheadOnDistThrd02; // m direction indicator ahead on distance threshould for exit begin
const float k_BA_MinDrctIndOnInAdvDst = 250.0f; // m if dist to waypoint is less than this threshold, then no indicator on in advance
const bool k_BA_LCDurTmInFlg = true; // determine whether to take lane change duration time into account
const float k_BA_RMMsgStopUpdtThr = 1.0f;
extern float k_BA_EffcTrffcCostSpdPerc01;
extern float k_BA_EffcTrffcCostSpdPerc02;
const float k_BA_SpprTOR3Dist2WpThr = 800.0f;
extern bool k_BA_AlwysLCR4SpltTypWP;
extern float k_BA_DistThr4LCR; // m
const float k_BA_SpprTOR3EndOffsetThr = 350.0f; // m
const float k_BA_SpprTOR3Dist2WpLowThr = 40.0f; // m
extern float k_BA_AllwAntiLCObjDstThr; // m
extern float k_BA_CnnctRmpSetSpdThr; // kph
extern float k_BA_Dst2FrtNrstOffsetThr; // m
extern float k_BA_MinFrtNrstLnLgthInCnctRmp; // m
extern float k_BA_CnnctRmpSetSpdThr01; // kph
extern float k_BA_Dst2FrtNrstOffsetThr01; // m
extern float k_BA_MinFrtNrstLnLgthInCnctRmp01; // m
extern float k_BA_PreviewTH4CurvCalc; // sec
extern float k_BA_AutoLCCrvHighThr; // m-1
extern float k_BA_AutoLCCrvLowThr; // m-1
extern bool k_BA_AutoLCCrvInFlg;
extern bool k_BA_LaneEndWrnInFlg;
extern bool k_BA_SmaNrrwLnInFlg;
extern float k_BA_SmaNrrwLnOffsetThrd01;
extern float k_BA_UpgradeDrvMdLnEndRngThr;
extern bool k_BA_ChgTRPLatStsAllw;
const int k_BA_TRPLatStsDebCnt = 5;
extern bool k_BA_InclLCP4SlowLnCost;
extern bool k_BA_AlphaVehFlowCalcAppl;
extern float k_BA_LowSpdRegionLowThr;
extern float k_BA_LowSpdRegionHighThr;
extern bool k_BA_ChgDrvMdWaitLongEnough;
extern float k_BA_ChgDrvMdWaitDurTmThr;
extern float k_BA_MinTHBtwFrtNrstObj;
extern float k_BA_SpdGap4AlphaVehFlowCalc;
const float k_BA_AntiLCPntThr = 1200.0f;
const bool k_BA_IsLCPAllw = true;
} // namespace common
} // namespace ADC
